@@ -2,7 +2,7 @@
    attmap.c - attribute mapping values and functions
    This file is part of the nss-pam-ldapd library.
 
-   Copyright (C) 2007, 2008, 2009 Arthur de Jong
+   Copyright (C) 2007, 2008, 2009, 2010, 2011 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -146,7 +146,7 @@ const char **attmap_get_var(enum ldap_map_selector map,const char *name)
     if (strcasecmp(name,"userPassword")==0)      return &attmap_group_userPassword;
     if (strcasecmp(name,"gidNumber")==0)         return &attmap_group_gidNumber;
     if (strcasecmp(name,"memberUid")==0)         return &attmap_group_memberUid;
-    if (strcasecmp(name,"uniqueMember")==0)      return &attmap_group_uniqueMember;
+    if (strcasecmp(name,"member")==0)            return &attmap_group_member;
   }
   else if (map==LM_HOSTS)
   {
@@ -213,10 +213,13 @@ const char *attmap_set_mapping(const char **var,const char *value)
     /* these attributes may contain an expression
        (note that this needs to match the functionality in the specific
        lookup module) */
-    if ( (var!=&attmap_passwd_gidNumber) &&
+    if ( (var!=&attmap_group_userPassword) &&
+         (var!=&attmap_passwd_userPassword) &&
+         (var!=&attmap_passwd_gidNumber) &&
          (var!=&attmap_passwd_gecos) &&
          (var!=&attmap_passwd_homeDirectory) &&
          (var!=&attmap_passwd_loginShell) &&
+         (var!=&attmap_shadow_userPassword) &&
          (var!=&attmap_shadow_shadowLastChange) &&
          (var!=&attmap_shadow_shadowMin) &&
          (var!=&attmap_shadow_shadowMax) &&
@@ -244,7 +247,7 @@ static const char *entry_expand(const char *name,void *expander_attr)
   /* TODO: handle userPassword attribute specially */
   if ((values[0]!=NULL)&&(values[1]!=NULL))
   {
-    log_log(LOG_WARNING,"entry %s contains multiple %s values",
+    log_log(LOG_WARNING,"%s: %s: multiple values",
                         myldap_get_dn(entry),name);
   }
   return values[0];
