@@ -1,7 +1,7 @@
 /*
    group.c - group entry lookup routines
-   Parts of this file were part of the nss_ldap library (as ldap-grp.c) which
-   has been forked into the nss-pam-ldapd library.
+   Parts of this file were part of the nss_ldap library (as ldap-grp.c)
+   which has been forked into the nss-pam-ldapd library.
 
    Copyright (C) 1997-2006 Luke Howard
    Copyright (C) 2006 West Consulting
@@ -37,18 +37,18 @@
 #include "myldap.h"
 #include "cfg.h"
 #include "attmap.h"
+#include "compat/strndup.h"
 
 /* ( nisSchema.2.2 NAME 'posixGroup' SUP top STRUCTURAL
  *   DESC 'Abstraction of a group of accounts'
  *   MUST ( cn $ gidNumber )
  *   MAY ( userPassword $ memberUid $ description ) )
  *
- * apart from that the above the uniqueMember attributes may be
- * supported in a coming release (they map to DNs, which is an extra
- * lookup step)
+ * apart from the above a member attribute is also supported that
+ * may contains a DN of a user
  *
- * using nested groups (groups that are member of a group) is currently
- * not supported, this may be added in a later release
+ * nested groups (groups that are member of a group) are currently
+ * not supported
  */
 
 /* the search base for searches */
@@ -62,17 +62,20 @@ const char *group_filter = "(objectClass=posixGroup)";
 
 /* the attributes to request with searches */
 const char *attmap_group_cn            = "cn";
-const char *attmap_group_userPassword  = "userPassword";
+const char *attmap_group_userPassword  = "\"*\"";
 const char *attmap_group_gidNumber     = "gidNumber";
 const char *attmap_group_memberUid     = "memberUid";
-const char *attmap_group_uniqueMember  = "uniqueMember";
+const char *attmap_group_member        = "member";
+
+/* special property for objectSid-based searches
+   (these are already LDAP-escaped strings) */
+static char *gidSid=NULL;
 
 /* default values for attributes */
 static const char *default_group_userPassword     = "*"; /* unmatchable */
 
-
 /* the attribute list to request with searches */
-static const char *group_attrs[6];
+static const char **group_attrs=NULL;
 
 /* create a search filter for searching a group entry
    by name, return -1 on errors */
@@ -95,10 +98,22 @@ static int mkfilter_group_byname(const char *name,
 static int mkfilter_group_bygid(gid_t gid,
                                 char *buffer,size_t buflen)
 {
-  return mysnprintf(buffer,buflen,
-                    "(&%s(%s=%d))",
-                    group_filter,
-                    attmap_group_gidNumber,(int)gid);
+  if (gidSid!=NULL)
+  {
+    return mysnprintf(buffer,buflen,
+                      "(&%s(%s=%s\\%02x\\%02x\\%02x\\%02x))",
+                      group_filter,
+                      attmap_group_gidNumber,gidSid,
+                      (int)(gid&0xff),(int)((gid>>8)&0xff),
+                      (int)((gid>>16)&0xff),(int)((gid>>24)&0xff));
+  }
+  else
+  {
+    return mysnprintf(buffer,buflen,
+                      "(&%s(%s=%d))",
+                      group_filter,
+                      attmap_group_gidNumber,(int)gid);
+  }
 }
 
 /* create a search filter for searching a group entry
@@ -127,12 +142,13 @@ static int mkfilter_group_bymember(MYLDAP_SESSION *session,
                     "(&%s(|(%s=%s)(%s=%s)))",
                     group_filter,
                     attmap_group_memberUid,safeuid,
-                    attmap_group_uniqueMember,safedn);
+                    attmap_group_member,safedn);
 }
 
 void group_init(void)
 {
   int i;
+  SET *set;
   /* set up search bases */
   if (group_bases[0]==NULL)
     for (i=0;i<NSS_LDAP_CONFIG_MAX_BASES;i++)
@@ -140,13 +156,21 @@ void group_init(void)
   /* set up scope */
   if (group_scope==LDAP_SCOPE_DEFAULT)
     group_scope=nslcd_cfg->ldc_scope;
+  /* special case when gidNumber references objectSid */
+  if (strncasecmp(attmap_group_gidNumber,"objectSid:",10)==0)
+  {
+    gidSid=sid2search(attmap_group_gidNumber+10);
+    attmap_group_gidNumber=strndup(attmap_group_gidNumber,9);
+  }
   /* set up attribute list */
-  group_attrs[0]=attmap_group_cn;
-  group_attrs[1]=attmap_group_userPassword;
-  group_attrs[2]=attmap_group_memberUid;
-  group_attrs[3]=attmap_group_gidNumber;
-  group_attrs[4]=attmap_group_uniqueMember;
-  group_attrs[5]=NULL;
+  set=set_new();
+  attmap_add_attributes(set,attmap_group_cn);
+  attmap_add_attributes(set,attmap_group_userPassword);
+  attmap_add_attributes(set,attmap_group_memberUid);
+  attmap_add_attributes(set,attmap_group_gidNumber);
+  attmap_add_attributes(set,attmap_group_member);
+  group_attrs=set_tolist(set);
+  set_free(set);
 }
 
 static int do_write_group(
@@ -160,8 +184,8 @@ static int do_write_group(
   {
     if (!isvalidname(names[i]))
     {
-      log_log(LOG_WARNING,"group entry %s contains invalid group name: \"%s\"",
-                          myldap_get_dn(entry),names[i]);
+      log_log(LOG_WARNING,"%s: %s: denied by validnames option",
+                          myldap_get_dn(entry),attmap_group_cn);
     }
     else if ((reqname==NULL)||(strcmp(reqname,names[i])==0))
     {
@@ -197,8 +221,8 @@ static const char **getmembers(MYLDAP_ENTRY *entry,MYLDAP_SESSION *session)
       if (isvalidname(values[i]))
         set_add(set,values[i]);
     }
-  /* add the uniqueMember values */
-  values=myldap_get_values(entry,attmap_group_uniqueMember);
+  /* add the member values */
+  values=myldap_get_values(entry,attmap_group_member);
   if (values!=NULL)
     for (i=0;values[i]!=NULL;i++)
     {
@@ -225,12 +249,13 @@ static int write_group(TFILE *fp,MYLDAP_ENTRY *entry,const char *reqname,
   gid_t gids[MAXGIDS_PER_ENTRY];
   int numgids;
   char *tmp;
+  char passbuffer[64];
   int rc;
   /* get group name (cn) */
   names=myldap_get_values(entry,attmap_group_cn);
   if ((names==NULL)||(names[0]==NULL))
   {
-    log_log(LOG_WARNING,"group entry %s does not contain %s value",
+    log_log(LOG_WARNING,"%s: %s: missing",
                         myldap_get_dn(entry),attmap_group_cn);
     return 0;
   }
@@ -242,29 +267,41 @@ static int write_group(TFILE *fp,MYLDAP_ENTRY *entry,const char *reqname,
   }
   else
   {
-    gidvalues=myldap_get_values(entry,attmap_group_gidNumber);
+    gidvalues=myldap_get_values_len(entry,attmap_group_gidNumber);
     if ((gidvalues==NULL)||(gidvalues[0]==NULL))
     {
-      log_log(LOG_WARNING,"group entry %s does not contain %s value",
+      log_log(LOG_WARNING,"%s: %s: missing",
                           myldap_get_dn(entry),attmap_group_gidNumber);
       return 0;
     }
-    for (numgids=0;(gidvalues[numgids]!=NULL)&&(numgids<MAXGIDS_PER_ENTRY);numgids++)
+    for (numgids=0;(numgids<MAXGIDS_PER_ENTRY)&&(gidvalues[numgids]!=NULL);numgids++)
     {
-      gids[numgids]=(gid_t)strtol(gidvalues[numgids],&tmp,0);
-      if ((*(gidvalues[numgids])=='\0')||(*tmp!='\0'))
+      if (gidSid!=NULL)
+        gids[numgids]=(gid_t)binsid2id(gidvalues[numgids]);
+      else
       {
-        log_log(LOG_WARNING,"group entry %s contains non-numeric %s value",
-                            myldap_get_dn(entry),attmap_group_gidNumber);
-        return 0;
+        errno=0;
+        gids[numgids]=strtogid(gidvalues[numgids],&tmp,10);
+        if ((*(gidvalues[numgids])=='\0')||(*tmp!='\0'))
+        {
+          log_log(LOG_WARNING,"%s: %s: non-numeric",
+                              myldap_get_dn(entry),attmap_group_gidNumber);
+          return 0;
+        }
+        else if (errno!=0)
+        {
+          log_log(LOG_WARNING,"%s: %s: too large",
+                              myldap_get_dn(entry),attmap_group_gidNumber);
+          return 0;
+        }
       }
     }
   }
   /* get group passwd (userPassword) (use only first entry) */
-  passwd=get_userpassword(entry,attmap_group_userPassword);
+  passwd=get_userpassword(entry,attmap_group_userPassword,passbuffer,sizeof(passbuffer));
   if (passwd==NULL)
     passwd=default_group_userPassword;
-  /* get group memebers (memberUid&uniqueMember) */
+  /* get group memebers (memberUid&member) */
   if (wantmembers)
     members=getmembers(entry,session);
   else
@@ -283,11 +320,11 @@ NSLCD_HANDLE(
   char name[256];
   char filter[1024];
   READ_STRING(fp,name);
+  log_setrequest("group=\"%s\"",name);
   if (!isvalidname(name)) {
-    log_log(LOG_WARNING,"nslcd_group_byname(%s): invalid group name",name);
+    log_log(LOG_WARNING,"request denied by validnames option");
     return -1;
   },
-  log_log(LOG_DEBUG,"nslcd_group_byname(%s)",name);,
   NSLCD_ACTION_GROUP_BYNAME,
   mkfilter_group_byname(name,filter,sizeof(filter)),
   write_group(fp,entry,name,NULL,1,session)
@@ -297,8 +334,8 @@ NSLCD_HANDLE(
   group,bygid,
   gid_t gid;
   char filter[1024];
-  READ_TYPE(fp,gid,gid_t);,
-  log_log(LOG_DEBUG,"nslcd_group_bygid(%d)",(int)gid);,
+  READ_TYPE(fp,gid,gid_t);
+  log_setrequest("group=%d",(int)gid);,
   NSLCD_ACTION_GROUP_BYGID,
   mkfilter_group_bygid(gid,filter,sizeof(filter)),
   write_group(fp,entry,NULL,&gid,1,session)
@@ -309,20 +346,22 @@ NSLCD_HANDLE(
   char name[256];
   char filter[1024];
   READ_STRING(fp,name);
-  if (!isvalidname(name)) {
-    log_log(LOG_WARNING,"nslcd_group_bymember(%s): invalid user name",name);
+  log_setrequest("group/member=\"%s\"",name);
+  if (!isvalidname(name))
+  {
+    log_log(LOG_WARNING,"request denied by validnames option");
     return -1;
   }
   if ((nslcd_cfg->ldc_nss_initgroups_ignoreusers!=NULL)&&
       set_contains(nslcd_cfg->ldc_nss_initgroups_ignoreusers,name))
   {
+    log_log(LOG_DEBUG,"ignored group member");
     /* just end the request, returning no results */
     WRITE_INT32(fp,NSLCD_VERSION);
     WRITE_INT32(fp,NSLCD_ACTION_GROUP_BYMEMBER);
     WRITE_INT32(fp,NSLCD_RESULT_END);
     return 0;
   },
-  log_log(LOG_DEBUG,"nslcd_group_bymember(%s)",name);,
   NSLCD_ACTION_GROUP_BYMEMBER,
   mkfilter_group_bymember(session,name,filter,sizeof(filter)),
   write_group(fp,entry,NULL,NULL,0,session)
@@ -331,8 +370,7 @@ NSLCD_HANDLE(
 NSLCD_HANDLE(
   group,all,
   const char *filter;
-  /* no parameters to read */,
-  log_log(LOG_DEBUG,"nslcd_group_all()");,
+  log_setrequest("group(all)");,
   NSLCD_ACTION_GROUP_ALL,
   (filter=group_filter,0),
   write_group(fp,entry,NULL,NULL,1,session)
