@@ -314,7 +314,7 @@ static int create_socket(const char *filename)
   strncpy(addr.sun_path,filename,sizeof(addr.sun_path));
   addr.sun_path[sizeof(addr.sun_path)-1]='\0';
   /* bind to the named socket */
-  if (bind(sock,(struct sockaddr *)&addr,(sizeof(addr.sun_family)+strlen(addr.sun_path))))
+  if (bind(sock,(struct sockaddr *)&addr,SUN_LEN(&addr)))
   {
     log_log(LOG_ERR,"bind() to %s failed: %s",filename,strerror(errno));
     if (close(sock))
@@ -324,7 +324,7 @@ static int create_socket(const char *filename)
   /* close the file descriptor on exit */
   if (fcntl(sock,F_SETFD,FD_CLOEXEC)<0)
   {
-    log_log(LOG_ERR,"fctnl(F_SETFL,FD_CLOEXEC) failed: %s",strerror(errno));
+    log_log(LOG_ERR,"fctnl(F_SETFL,FD_CLOEXEC) on %s failed: %s",filename,strerror(errno));
     if (close(sock))
       log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
     exit(EXIT_FAILURE);
@@ -336,7 +336,7 @@ static int create_socket(const char *filename)
      http://lkml.org/lkml/2005/5/16/11 */
   if (chmod(filename,(mode_t)0666))
   {
-    log_log(LOG_ERR,"chmod(0666) failed: %s",strerror(errno));
+    log_log(LOG_ERR,"chmod(0666) of %s failed: %s",filename,strerror(errno));
     if (close(sock))
       log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
     exit(EXIT_FAILURE);
@@ -472,10 +472,12 @@ static int is_locked(const char* filename)
     }
     if (lockf(fd,F_TEST,0)<0)
     {
-      close(fd);
+      if (close(fd))
+        log_log(LOG_WARNING,"problem closing fd: %s",strerror(errno));
       return -1;
     }
-    close(fd);
+    if (close(fd))
+      log_log(LOG_WARNING,"problem closing fd: %s",strerror(errno));
   }
   return 0;
 }
@@ -598,7 +600,7 @@ static void *worker(void UNUSED(*arg))
         log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
       continue;
     }
-    /* indicate new connection to logging module (genrates unique id) */
+    /* indicate new connection to logging module (generates unique id) */
     log_newsession();
     /* handle the connection */
     handleconnection(csock,session);
