@@ -1,11 +1,11 @@
 /*
-   protocol.c - network address entry lookup routines
+   protocol.c - protocol name and number lookup routines
    Parts of this file were part of the nss_ldap library (as ldap-proto.c)
    which has been forked into the nss-pam-ldapd library.
 
    Copyright (C) 1997-2005 Luke Howard
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2009, 2010 Arthur de Jong
+   Copyright (C) 2006, 2007, 2009, 2010, 2011 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -115,7 +115,7 @@ static int write_protocol(TFILE *fp,MYLDAP_ENTRY *entry,const char *reqname)
   aliases=myldap_get_values(entry,attmap_protocol_cn);
   if ((aliases==NULL)||(aliases[0]==NULL))
   {
-    log_log(LOG_WARNING,"protocol entry %s does not contain %s value",
+    log_log(LOG_WARNING,"%s: %s: missing",
                         myldap_get_dn(entry),attmap_protocol_cn);
     return 0;
   }
@@ -134,19 +134,26 @@ static int write_protocol(TFILE *fp,MYLDAP_ENTRY *entry,const char *reqname)
   protos=myldap_get_values(entry,attmap_protocol_ipProtocolNumber);
   if ((protos==NULL)||(protos[0]==NULL))
   {
-    log_log(LOG_WARNING,"protocol entry %s does not contain %s value",
+    log_log(LOG_WARNING,"%s: %s: missing",
                         myldap_get_dn(entry),attmap_protocol_ipProtocolNumber);
     return 0;
   }
   else if (protos[1]!=NULL)
   {
-    log_log(LOG_WARNING,"protocol entry %s contains multiple %s values",
+    log_log(LOG_WARNING,"%s: %s: multiple values",
                         myldap_get_dn(entry),attmap_protocol_ipProtocolNumber);
   }
-  proto=(int)strtol(protos[0],&tmp,0);
+  errno=0;
+  proto=(int)strtol(protos[0],&tmp,10);
   if ((*(protos[0])=='\0')||(*tmp!='\0'))
   {
-    log_log(LOG_WARNING,"protocol entry %s contains non-numeric %s value",
+    log_log(LOG_WARNING,"%s: %s: non-numeric",
+                        myldap_get_dn(entry),attmap_protocol_ipProtocolNumber);
+    return 0;
+  }
+  else if (errno!=0)
+  {
+    log_log(LOG_WARNING,"%s: %s: too large",
                         myldap_get_dn(entry),attmap_protocol_ipProtocolNumber);
     return 0;
   }
@@ -162,8 +169,8 @@ NSLCD_HANDLE(
   protocol,byname,
   char name[256];
   char filter[1024];
-  READ_STRING(fp,name);,
-  log_log(LOG_DEBUG,"nslcd_protocol_byname(%s)",name);,
+  READ_STRING(fp,name);
+  log_setrequest("protocol=\"%s\"",name);,
   NSLCD_ACTION_PROTOCOL_BYNAME,
   mkfilter_protocol_byname(name,filter,sizeof(filter)),
   write_protocol(fp,entry,name)
@@ -173,8 +180,8 @@ NSLCD_HANDLE(
   protocol,bynumber,
   int protocol;
   char filter[1024];
-  READ_INT32(fp,protocol);,
-  log_log(LOG_DEBUG,"nslcd_protocol_bynumber(%d)",protocol);,
+  READ_INT32(fp,protocol);
+  log_setrequest("protocol=%d",protocol);,
   NSLCD_ACTION_PROTOCOL_BYNUMBER,
   mkfilter_protocol_bynumber(protocol,filter,sizeof(filter)),
   write_protocol(fp,entry,NULL)
@@ -183,8 +190,7 @@ NSLCD_HANDLE(
 NSLCD_HANDLE(
   protocol,all,
   const char *filter;
-  /* no parameters to read */,
-  log_log(LOG_DEBUG,"nslcd_protocol_all()");,
+  log_setrequest("protocol(all)");,
   NSLCD_ACTION_PROTOCOL_ALL,
   (filter=protocol_filter,0),
   write_protocol(fp,entry,NULL)
