@@ -1,11 +1,11 @@
 /*
-   shadow.c - service entry lookup routines
+   shadow.c - shadow entry lookup routines
    Parts of this file were part of the nss_ldap library (as ldap-spwd.c)
    which has been forked into the nss-pam-ldapd library.
 
    Copyright (C) 1997-2005 Luke Howard
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2009 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -28,6 +28,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 
 #include "common.h"
 #include "log.h"
@@ -54,7 +55,7 @@ const char *shadow_filter = "(objectClass=shadowAccount)";
 
 /* the attributes to request with searches */
 const char *attmap_shadow_uid              = "uid";
-const char *attmap_shadow_userPassword     = "userPassword";
+const char *attmap_shadow_userPassword     = "\"*\"";
 const char *attmap_shadow_shadowLastChange = "\"${shadowLastChange:--1}\"";
 const char *attmap_shadow_shadowMin        = "\"${shadowMin:--1}\"";
 const char *attmap_shadow_shadowMax        = "\"${shadowMax:--1}\"";
@@ -72,7 +73,7 @@ static const char **shadow_attrs=NULL;
 static int mkfilter_shadow_byname(const char *name,
                                   char *buffer,size_t buflen)
 {
-  char safename[1024];
+  char safename[300];
   /* escape attribute */
   if(myldap_escape(name,safename,sizeof(safename)))
     return -1;
@@ -109,9 +110,9 @@ void shadow_init(void)
   set_free(set);
 }
 
-static long to_date(const char *date,const char *attr)
+static long to_date(const char *dn,const char *date,const char *attr)
 {
-  char buffer[8];
+  char buffer[32];
   long value;
   char *tmp;
   size_t l;
@@ -125,24 +126,36 @@ static long to_date(const char *date,const char *attr)
        last 9 digits from the string and going from there */
     l=strlen(date)-9;
     if (l>(sizeof(buffer)-1))
-      return 0; /* error */
+      return -1; /* error */
     strncpy(buffer,date,l);
     buffer[l]='\0';
-    value=strtol(date,&tmp,0);
+    errno=0;
+    value=strtol(date,&tmp,10);
     if ((*date=='\0')||(*tmp!='\0'))
     {
-      log_log(LOG_WARNING,"shadow entry contains non-numeric %s value",attr);
-      return 0;
+      log_log(LOG_WARNING,"%s: %s: non-numeric",dn,attr);
+      return -1;
+    }
+    else if (errno!=0)
+    {
+      log_log(LOG_WARNING,"%s: %s: too large",dn,attr);
+      return -1;
     }
     return value/864-134774;
     /* note that AD does not have expiry dates but a lastchangeddate
        and some value that needs to be added */
   }
-  value=strtol(date,&tmp,0);
+  errno=0;
+  value=strtol(date,&tmp,10);
   if ((*date=='\0')||(*tmp!='\0'))
   {
-    log_log(LOG_WARNING,"shadow entry contains non-numeric %s value",attr);
-    return 0;
+    log_log(LOG_WARNING,"%s: %s: non-numeric",dn,attr);
+    return -1;
+  }
+  else if (errno!=0)
+  {
+    log_log(LOG_WARNING,"%s: %s: too large",dn,attr);
+    return -1;
   }
   return value;
 }
@@ -151,29 +164,121 @@ static long to_date(const char *date,const char *attr)
 #define UF_DONT_EXPIRE_PASSWD 0x10000
 #endif
 
-#define GET_OPTIONAL_LONG(var,att) \
+#define GET_OPTIONAL_LONG(var,att,fallback) \
   tmpvalue=attmap_get_value(entry,attmap_shadow_##att,buffer,sizeof(buffer)); \
   if (tmpvalue==NULL) \
     tmpvalue=""; \
-  var=strtol(tmpvalue,&tmp,0); \
+  errno=0; \
+  var=strtol(tmpvalue,&tmp,10); \
   if ((*(tmpvalue)=='\0')||(*tmp!='\0')) \
   { \
-    log_log(LOG_WARNING,"shadow entry %s contains non-numeric %s value", \
+    log_log(LOG_WARNING,"%s: %s: non-numeric", \
                         myldap_get_dn(entry),attmap_shadow_##att); \
-    return 0; \
+    var=fallback; \
+  } \
+  else if (errno!=0) \
+  { \
+    log_log(LOG_WARNING,"%s: %s: too large", \
+                        myldap_get_dn(entry),attmap_shadow_##att); \
+    var=fallback; \
   }
 
-#define GET_OPTIONAL_DATE(var,att) \
-  tmpvalue=attmap_get_value(entry,attmap_shadow_##att,buffer,sizeof(buffer)); \
-  if (tmpvalue==NULL) \
-    tmpvalue=""; \
-  var=to_date(tmpvalue,attmap_shadow_##att);
+void get_shadow_properties(MYLDAP_ENTRY *entry,long *lastchangedate,
+                           long *mindays,long *maxdays,long *warndays,
+                           long *inactdays,long *expiredate,unsigned long *flag)
+{
+  char buffer[64];
+  const char *tmpvalue;
+  char *tmp;
+  /* get lastchange date */
+  tmpvalue=attmap_get_value(entry,attmap_shadow_shadowLastChange,buffer,sizeof(buffer));
+  if (tmpvalue==NULL)
+    tmpvalue="";
+  *lastchangedate=to_date(myldap_get_dn(entry),tmpvalue,attmap_shadow_shadowLastChange);
+  /* get other shadow properties */
+  GET_OPTIONAL_LONG(*mindays,shadowMin,-1);
+  GET_OPTIONAL_LONG(*maxdays,shadowMax,-1);
+  GET_OPTIONAL_LONG(*warndays,shadowWarning,-1);
+  GET_OPTIONAL_LONG(*inactdays,shadowInactive,-1);
+  GET_OPTIONAL_LONG(*expiredate,shadowExpire,-1);
+  GET_OPTIONAL_LONG(*flag,shadowFlag,0);
+  /* if we're using AD handle the flag specially */
+  if (strcasecmp(attmap_shadow_shadowLastChange,"pwdLastSet")==0)
+  {
+    if (*flag&UF_DONT_EXPIRE_PASSWD)
+      *maxdays=-1;
+    *flag=0;
+  }
+}
+
+/* try to update the shadowLastChange attribute of the entry if possible */
+int update_lastchange(MYLDAP_SESSION *session,const char *userdn)
+{
+  MYLDAP_SEARCH *search;
+  MYLDAP_ENTRY *entry;
+  static const char *attrs[3];
+  const char *attr;
+  int rc;
+  const char **values;
+  LDAPMod mod,*mods[2];
+  char buffer[64],*strvals[2];
+  /* find the name of the attribute to use */
+  if ( (attmap_shadow_shadowLastChange==NULL) || (attmap_shadow_shadowLastChange[0]=='\0') )
+    return LDAP_LOCAL_ERROR; /* attribute not mapped at all */
+  else if (strcmp(attmap_shadow_shadowLastChange,"\"${shadowLastChange:--1}\"")==0)
+    attr="shadowLastChange";
+  else if (attmap_shadow_shadowLastChange[0]=='\"')
+    return LDAP_LOCAL_ERROR; /* other expressions not supported for now */
+  else
+    attr=attmap_shadow_shadowLastChange;
+  /* set up the attributes we need */
+  attrs[0]=attmap_shadow_uid;
+  attrs[1]=attr;
+  attrs[2]=NULL;
+  /* find the entry to see if the attribute is present */
+  search=myldap_search(session,userdn,LDAP_SCOPE_BASE,shadow_filter,attrs,&rc);
+  if (search==NULL)
+    return rc;
+  entry=myldap_get_entry(search,&rc);
+  if (entry==NULL)
+    return rc;
+  values=myldap_get_values(entry,attr);
+  if ((values==NULL)||(values[0]==NULL)||(values[0][0]=='\0'))
+    return LDAP_NO_SUCH_ATTRIBUTE;
+  /* build the value for the new attribute */
+  if (strcasecmp(attr,"pwdLastSet")==0)
+  {
+    /* for AD we use another timestamp */
+    if(mysnprintf(buffer,sizeof(buffer),"%ld000000000",((long int)time(NULL)/100L+(134774L*864L))))
+      return LDAP_LOCAL_ERROR;
+  }
+  else
+  {
+    /* time in days since Jan 1, 1970 */
+    if(mysnprintf(buffer,sizeof(buffer),"%ld",((long int)(time(NULL)/(long int)(60*60*24)))))
+      return LDAP_LOCAL_ERROR;
+  }
+  /* update the shadowLastChange attribute */
+  strvals[0]=buffer;
+  strvals[1]=NULL;
+  mod.mod_op=LDAP_MOD_REPLACE;
+  mod.mod_type=(char *)attr;
+  mod.mod_values=strvals;
+  mods[0]=&mod;
+  mods[1]=NULL;
+  rc=myldap_modify(session,userdn,mods);
+  if (rc!=LDAP_SUCCESS)
+    log_log(LOG_WARNING,"%s: %s: modification failed: %s",
+                        userdn,attr,ldap_err2string(rc));
+  else
+    log_log(LOG_DEBUG,"%s: %s: modification succeeded",
+                      userdn,attr);
+  return rc;
+}
 
 static int write_shadow(TFILE *fp,MYLDAP_ENTRY *entry,const char *requser)
 {
   int32_t tmpint32;
-  const char *tmpvalue;
-  char *tmp;
   const char **usernames;
   const char *passwd;
   long lastchangedate;
@@ -184,40 +289,22 @@ static int write_shadow(TFILE *fp,MYLDAP_ENTRY *entry,const char *requser)
   long expiredate;
   unsigned long flag;
   int i;
-  char buffer[80];
+  char passbuffer[64];
   /* get username */
   usernames=myldap_get_values(entry,attmap_shadow_uid);
   if ((usernames==NULL)||(usernames[0]==NULL))
   {
-    log_log(LOG_WARNING,"shadow entry %s does not contain %s value",
+    log_log(LOG_WARNING,"%s: %s: missing",
                         myldap_get_dn(entry),attmap_shadow_uid);
     return 0;
   }
   /* get password */
-  passwd=get_userpassword(entry,attmap_shadow_userPassword);
+  passwd=get_userpassword(entry,attmap_shadow_userPassword,passbuffer,sizeof(passbuffer));
   if (passwd==NULL)
     passwd=default_shadow_userPassword;
-  /* get lastchange date */
-  GET_OPTIONAL_DATE(lastchangedate,shadowLastChange);
-  /* get mindays */
-  GET_OPTIONAL_LONG(mindays,shadowMin);
-  /* get maxdays */
-  GET_OPTIONAL_LONG(maxdays,shadowMax);
-  /* get warndays */
-  GET_OPTIONAL_LONG(warndays,shadowWarning);
-  /* get inactdays */
-  GET_OPTIONAL_LONG(inactdays,shadowInactive);
-  /* get expire date */
-  GET_OPTIONAL_LONG(expiredate,shadowExpire);
-  /* get flag */
-  GET_OPTIONAL_LONG(flag,shadowFlag);
-  /* if we're using AD handle the flag specially */
-  if (strcasecmp(attmap_shadow_shadowLastChange,"pwdLastSet")==0)
-  {
-    if (flag&UF_DONT_EXPIRE_PASSWD)
-      maxdays=99999;
-    flag=0;
-  }
+  /* get expiry properties */
+  get_shadow_properties(entry,&lastchangedate,&mindays,&maxdays,&warndays,
+                        &inactdays,&expiredate,&flag);
   /* write the entries */
   for (i=0;usernames[i]!=NULL;i++)
     if ((requser==NULL)||(strcmp(requser,usernames[i])==0))
@@ -236,12 +323,46 @@ static int write_shadow(TFILE *fp,MYLDAP_ENTRY *entry,const char *requser)
   return 0;
 }
 
+MYLDAP_ENTRY *shadow_uid2entry(MYLDAP_SESSION *session,const char *username,int *rcp)
+{
+  MYLDAP_SEARCH *search=NULL;
+  MYLDAP_ENTRY *entry=NULL;
+  const char *base;
+  char filter[1024];
+  int i;
+  /* if it isn't a valid username, just bail out now */
+  if (!isvalidname(username))
+  {
+    if (rcp!=NULL)
+      *rcp=LDAP_INVALID_SYNTAX;
+    return NULL;
+  }
+  /* we have to look up the entry */
+  mkfilter_shadow_byname(username,filter,sizeof(filter));
+  for (i=0;(i<NSS_LDAP_CONFIG_MAX_BASES)&&((base=shadow_bases[i])!=NULL);i++)
+  {
+    search=myldap_search(session,base,shadow_scope,filter,shadow_attrs,rcp);
+    if (search==NULL)
+    {
+      if ((rcp!=NULL)&&(*rcp==LDAP_SUCCESS))
+        *rcp=LDAP_NO_SUCH_OBJECT;
+      return NULL;
+    }
+    entry=myldap_get_entry(search,rcp);
+    if (entry!=NULL)
+      return entry;
+  }
+  if ((rcp!=NULL)&&(*rcp==LDAP_SUCCESS))
+    *rcp=LDAP_NO_SUCH_OBJECT;
+  return NULL;
+}
+
 NSLCD_HANDLE(
   shadow,byname,
   char name[256];
   char filter[1024];
-  READ_STRING(fp,name);,
-  log_log(LOG_DEBUG,"nslcd_shadow_byname(%s)",name);,
+  READ_STRING(fp,name);
+  log_setrequest("shadow=\"%s\"",name);,
   NSLCD_ACTION_SHADOW_BYNAME,
   mkfilter_shadow_byname(name,filter,sizeof(filter)),
   write_shadow(fp,entry,name)
@@ -250,8 +371,7 @@ NSLCD_HANDLE(
 NSLCD_HANDLE(
   shadow,all,
   const char *filter;
-  /* no parameters to read */,
-  log_log(LOG_DEBUG,"nslcd_shadow_all()");,
+  log_setrequest("shadow(all)");,
   NSLCD_ACTION_SHADOW_ALL,
   (filter=shadow_filter,0),
   write_shadow(fp,entry,NULL)
