@@ -2,7 +2,7 @@
    pam.c - pam module functions
 
    Copyright (C) 2009 Howard Chu
-   Copyright (C) 2009, 2010 Arthur de Jong
+   Copyright (C) 2009, 2010, 2011 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -92,7 +92,7 @@ static void ctx_clear(struct pld_ctx *ctx)
 }
 
 /* free the context (this is installed as handler into PAM) */
-static void ctx_free(pam_handle_t *UNUSED(pamh),void *data,int UNUSED(err))
+static void ctx_free(pam_handle_t UNUSED(*pamh),void *data,int UNUSED(err))
 {
   struct pld_ctx *ctx=data;
   ctx_clear(ctx);
@@ -138,8 +138,7 @@ static int ctx_get(pam_handle_t *pamh,const char *username,struct pld_ctx **pctx
 
 /* our PAM module configuration */
 struct pld_cfg {
-  int use_first_pass;
-  int try_first_pass;
+  int nullok;
   int no_warn;
   int ignore_unknown_user;
   int ignore_authinfo_unavail;
@@ -147,16 +146,12 @@ struct pld_cfg {
   uid_t minimum_uid;
 };
 
-static int init(pam_handle_t *pamh,int flags,int argc,const char **argv,
-                struct pld_cfg *cfg,struct pld_ctx **ctx,const char **username,
-                const char **service)
+static void cfg_init(pam_handle_t *pamh,int flags,int argc,const char **argv,
+                     struct pld_cfg *cfg)
 {
   int i;
-  int rc;
-  struct passwd *pwent;
   /* initialise config with defaults */
-  cfg->use_first_pass=0;
-  cfg->try_first_pass=0;
+  cfg->nullok=0;
   cfg->no_warn=0;
   cfg->ignore_unknown_user=0;
   cfg->ignore_authinfo_unavail=0;
@@ -166,9 +161,11 @@ static int init(pam_handle_t *pamh,int flags,int argc,const char **argv,
   for (i=0;i<argc;i++)
   {
     if (strcmp(argv[i],"use_first_pass")==0)
-      cfg->use_first_pass=1;
+      /* ignore, this option is used by pam_get_authtok() internally */;
     else if (strcmp(argv[i],"try_first_pass")==0)
-      cfg->try_first_pass=1;
+      /* ignore, this option is used by pam_get_authtok() internally */;
+    else if (strcmp(argv[i],"nullok")==0)
+      cfg->nullok=1;
     else if (strcmp(argv[i],"use_authtok")==0)
       /* ignore, this option is used by pam_get_authtok() internally */;
     else if (strcmp(argv[i],"no_warn")==0)
@@ -187,6 +184,13 @@ static int init(pam_handle_t *pamh,int flags,int argc,const char **argv,
   /* check flags */
   if (flags&PAM_SILENT)
     cfg->no_warn=1;
+}
+
+static int init(pam_handle_t *pamh,struct pld_cfg *cfg,struct pld_ctx **ctx,
+                const char **username,const char **service)
+{
+  int rc;
+  struct passwd *pwent;
   /* get user name */
   rc=pam_get_user(pamh,username,NULL);
   if (rc!=PAM_SUCCESS)
@@ -225,7 +229,7 @@ static int init(pam_handle_t *pamh,int flags,int argc,const char **argv,
 }
 
 /* map a NSLCD PAM status code to a PAM status code */
-static int nslcd2pam_rc(int rc)
+static int nslcd2pam_rc(pam_handle_t *pamh,int rc)
 {
 #define map(i) case NSLCD_##i: return i;
   switch(rc) {
@@ -239,13 +243,39 @@ static int nslcd2pam_rc(int rc)
     map(PAM_NEW_AUTHTOK_REQD);
     map(PAM_ACCT_EXPIRED);
     map(PAM_SESSION_ERR);
+    map(PAM_AUTHTOK_ERR);
     map(PAM_AUTHTOK_DISABLE_AGING);
     map(PAM_IGNORE);
     map(PAM_ABORT);
-    default: return PAM_ABORT;
+    map(PAM_AUTHTOK_EXPIRED);
+    default:
+      pam_syslog(pamh,LOG_ERR,"unknown NSLCD_PAM_* code returned: %d",rc);
+      return PAM_ABORT;
   }
 }
 
+/* check whether the specified user is handled by nslcd */
+static int nslcd_request_exists(pam_handle_t *pamh,struct pld_ctx *ctx,struct pld_cfg *cfg,
+                                const char *username)
+{
+  uid_t dummy_uid;
+  gid_t dummy_gid;
+  PAM_REQUEST(NSLCD_ACTION_PASSWD_BYNAME,
+    /* log debug message */
+    pam_syslog(pamh,LOG_DEBUG,"nslcd account check; user=%s",username),
+    /* write the request parameters */
+    WRITE_STRING(fp,username),
+    /* read the result entry */
+    SKIP_STRING(fp); /* user name */
+    SKIP_STRING(fp); /* passwd entry */
+    READ_TYPE(fp,dummy_uid,uid_t);
+    READ_TYPE(fp,dummy_gid,gid_t);
+    SKIP_STRING(fp); /* gecos */
+    SKIP_STRING(fp); /* home dir */
+    SKIP_STRING(fp); /* shell */
+  )
+}
+
 /* perform an authentication call over nslcd */
 static int nslcd_request_authc(pam_handle_t *pamh,struct pld_ctx *ctx,struct pld_cfg *cfg,
                                const char *username,const char *service,
@@ -299,7 +329,7 @@ static int nslcd_request_sess(pam_handle_t *pamh,struct pld_ctx *ctx,struct pld_
   PAM_REQUEST(action,
     /* log debug message */
     pam_syslog(pamh,LOG_DEBUG,"nslcd session %s; user=%s",
-          (action==NSLCD_ACTION_PAM_SESS_O)?"open":"clode",username),
+          (action==NSLCD_ACTION_PAM_SESS_O)?"open":"close",username),
     /* write the request parameters */
     WRITE_STRING(fp,username);
     WRITE_STRING(fp,ctx->dn);
@@ -333,6 +363,16 @@ static int nslcd_request_pwmod(pam_handle_t *pamh,struct pld_ctx *ctx,struct pld
     READ_BUF_STRING(fp,ctx->authzmsg);)
 }
 
+/* remap the return code based on the configuration */
+static int remap_pam_rc(int rc,struct pld_cfg *cfg)
+{
+  if ((rc==PAM_AUTHINFO_UNAVAIL)&&cfg->ignore_authinfo_unavail)
+    return PAM_IGNORE;
+  if ((rc==PAM_USER_UNKNOWN)&&cfg->ignore_unknown_user)
+    return PAM_IGNORE;
+  return rc;
+}
+
 /* PAM authentication check */
 int pam_sm_authenticate(pam_handle_t *pamh,int flags,int argc,const char **argv)
 {
@@ -341,60 +381,46 @@ int pam_sm_authenticate(pam_handle_t *pamh,int flags,int argc,const char **argv)
   struct pld_ctx *ctx;
   const char *username,*service;
   char *passwd=NULL;
-  int i;
   /* set up configuration */
-  rc=init(pamh,flags,argc,argv,&cfg,&ctx,&username,&service);
+  cfg_init(pamh,flags,argc,argv,&cfg);
+  rc=init(pamh,&cfg,&ctx,&username,&service);
   if (rc!=PAM_SUCCESS)
+    return remap_pam_rc(rc,&cfg);
+  /* get the password */
+  rc=pam_get_authtok(pamh,PAM_AUTHTOK,(const char **)&passwd,NULL);
+  if (rc!=PAM_SUCCESS)
+  {
+    pam_syslog(pamh,LOG_ERR,"failed to get password: %s",pam_strerror(pamh,rc));
     return rc;
-  /* try twice */
-  for (i=0;i<2;i++)
-  {
-    if ((!cfg.try_first_pass)&&(!cfg.use_first_pass))
-    {
-      rc=pam_get_authtok(pamh,PAM_AUTHTOK,(const char **)&passwd,i==0?"Password: ":"LDAP Password: ");
-      if (rc!=PAM_SUCCESS)
-      {
-        pam_syslog(pamh,LOG_ERR,"failed to get password: %s",pam_strerror(pamh,rc));
-        return rc;
-      }
-      /* exit loop after trying this password */
-      i=2;
-    }
-    rc=pam_get_item(pamh,PAM_AUTHTOK,(const void **)&passwd);
-    if (rc!=PAM_SUCCESS)
-      pam_syslog(pamh,LOG_ERR,"failed to get password: %s",pam_strerror(pamh,rc));
-    if (rc==PAM_SUCCESS)
-    {
-      rc=nslcd_request_authc(pamh,ctx,&cfg,username,service,passwd);
-      if (rc==PAM_SUCCESS)
-      {
-        rc=ctx->authok;
-        if (rc!=PAM_SUCCESS)
-          pam_syslog(pamh,LOG_NOTICE,"%s; user=%s",pam_strerror(pamh,rc),username);
-        else if (cfg.debug)
-          pam_syslog(pamh,LOG_DEBUG,"authentication succeeded");
-      }
-      /* remap error code */
-      if ((rc==PAM_AUTHINFO_UNAVAIL)&&cfg.ignore_authinfo_unavail)
-        rc=PAM_IGNORE;
-      else if ((rc==PAM_USER_UNKNOWN)&&cfg.ignore_unknown_user)
-        rc=PAM_IGNORE;
-    }
-    if ((rc==PAM_SUCCESS)||(cfg.use_first_pass))
-      break;
-    cfg.try_first_pass=0;
   }
+  /* check password */
+  if (!cfg.nullok&&((passwd==NULL)||(passwd[0]=='\0')))
+  {
+    if (cfg.debug)
+      pam_syslog(pamh,LOG_DEBUG,"user has empty password, access denied");
+    return PAM_AUTH_ERR;
+  }
+  /* do the nslcd request */
+  rc=nslcd_request_authc(pamh,ctx,&cfg,username,service,passwd);
+  if (rc!=PAM_SUCCESS)
+    return remap_pam_rc(rc,&cfg);
+  /* check the authentication result */
+  rc=ctx->authok;
+  if (rc!=PAM_SUCCESS)
+  {
+    pam_syslog(pamh,LOG_NOTICE,"%s; user=%s",pam_strerror(pamh,rc),username);
+    return remap_pam_rc(rc,&cfg);
+  }
+  /* debug log */
+  if (cfg.debug)
+    pam_syslog(pamh,LOG_DEBUG,"authentication succeeded");
   /* save username */
-  if (rc==PAM_SUCCESS)
-  {
-    ctx->user=strdup(username);
-    /* if password change is required, save old password in context */
-    if (ctx->authz==PAM_NEW_AUTHTOK_REQD)
-      ctx->oldpassword=strdup(passwd);
-  }
+  ctx->user=strdup(username);
+  /* if password change is required, save old password in context */
+  if (ctx->authz==PAM_NEW_AUTHTOK_REQD)
+    ctx->oldpassword=strdup(passwd);
   /* update caller's idea of the user name */
-  if ( (rc==PAM_SUCCESS) && ctx->tmpluser && ctx->tmpluser[0] &&
-       (strcmp(ctx->tmpluser,username)!=0) )
+  if ( ctx->tmpluser && ctx->tmpluser[0] && (strcmp(ctx->tmpluser,username)!=0) )
   {
     pam_syslog(pamh,LOG_INFO,"username changed from %s to %s",username,
                ctx->tmpluser);
@@ -420,9 +446,10 @@ int pam_sm_acct_mgmt(pam_handle_t *pamh,int flags,int argc,const char **argv)
   const char *username,*service;
   const char *ruser=NULL,*rhost=NULL,*tty=NULL;
   /* set up configuration */
-  rc=init(pamh,flags,argc,argv,&cfg,&ctx,&username,&service);
+  cfg_init(pamh,flags,argc,argv,&cfg);
+  rc=init(pamh,&cfg,&ctx,&username,&service);
   if (rc!=PAM_SUCCESS)
-    return rc;
+    return remap_pam_rc(rc,&cfg);
   /* get more PAM information */
   pam_get_item(pamh,PAM_RUSER,(const void **)&ruser);
   pam_get_item(pamh,PAM_RHOST,(const void **)&rhost);
@@ -431,34 +458,32 @@ int pam_sm_acct_mgmt(pam_handle_t *pamh,int flags,int argc,const char **argv)
      original context */
   ctx2.dn=ctx->dn;
   ctx2.user=ctx->user;
+  /* do the nslcd request */
   rc=nslcd_request_authz(pamh,&ctx2,&cfg,username,service,ruser,rhost,tty);
-  /* remap error code */
-  if ((rc==PAM_AUTHINFO_UNAVAIL)&&cfg.ignore_authinfo_unavail)
-    rc=PAM_IGNORE;
-  else if ((rc==PAM_USER_UNKNOWN)&&cfg.ignore_unknown_user)
-    rc=PAM_IGNORE;
   if (rc!=PAM_SUCCESS)
-  {
-    if (rc!=PAM_IGNORE)
-      if (!cfg.no_warn)
-        pam_error(pamh,"LDAP authorization failed");
-    return rc;
-  }
+    return remap_pam_rc(rc,&cfg);
   /* check the returned authorisation value */
   if (ctx2.authz!=PAM_SUCCESS)
   {
+    /* turn in to generic PAM error message if message is empty */
+    if ((ctx2.authzmsg==NULL)||(ctx2.authzmsg[0]=='\0'))
+      ctx2.authzmsg=(char *)pam_strerror(pamh,ctx2.authz);
     pam_syslog(pamh,LOG_NOTICE,"%s; user=%s",ctx2.authzmsg,username);
-    if (!cfg.no_warn)
+    rc=remap_pam_rc(ctx2.authz,&cfg);
+    if ((rc!=PAM_IGNORE)&&(!cfg.no_warn))
       pam_error(pamh,"%s",ctx2.authzmsg);
-    return ctx2.authz;
+    return rc;
   }
   /* check the original authorisation check from authentication */
   if (ctx->authz!=PAM_SUCCESS)
   {
+    if ((ctx->authzmsg==NULL)||(ctx->authzmsg[0]=='\0'))
+      ctx->authzmsg=(char *)pam_strerror(pamh,ctx->authz);
     pam_syslog(pamh,LOG_NOTICE,"%s; user=%s",ctx->authzmsg,username);
-    if (!cfg.no_warn)
+    rc=remap_pam_rc(ctx->authz,&cfg);
+    if ((rc!=PAM_IGNORE)&&(!cfg.no_warn))
       pam_error(pamh,"%s",ctx->authzmsg);
-    return ctx->authz;
+    return rc;
   }
   if (cfg.debug)
     pam_syslog(pamh,LOG_DEBUG,"authorization succeeded");
@@ -480,9 +505,10 @@ static int pam_sm_session(pam_handle_t *pamh,int flags,int argc,
   const char *username,*service;
   const char *tty=NULL,*rhost=NULL,*ruser=NULL;
   /* set up configuration */
-  rc=init(pamh,flags,argc,argv,&cfg,&ctx,&username,&service);
+  cfg_init(pamh,flags,argc,argv,&cfg);
+  rc=init(pamh,&cfg,&ctx,&username,&service);
   if (rc!=PAM_SUCCESS)
-    return rc;
+    return remap_pam_rc(rc,&cfg);
   /* get more PAM information */
   pam_get_item(pamh,PAM_TTY,(const void **)&tty);
   pam_get_item(pamh,PAM_RHOST,(const void **)&rhost);
@@ -490,20 +516,12 @@ static int pam_sm_session(pam_handle_t *pamh,int flags,int argc,
   /* do the nslcd request */
   rc=nslcd_request_sess(pamh,ctx,&cfg,action,username,service,tty,rhost,ruser);
   if (rc!=PAM_SUCCESS)
-    pam_syslog(pamh,LOG_NOTICE,"%s; user=%s",pam_strerror(pamh,rc),username);
-  else if (cfg.debug)
+    return remap_pam_rc(rc,&cfg);
+  /* debug log */
+  if (cfg.debug)
     pam_syslog(pamh,LOG_DEBUG,"session %s succeeded; session_id=%d",
-               (action==NSLCD_ACTION_PAM_SESS_O)?"open":"clode",ctx->sessid);
-  /* remap error code */
-  if ((rc==PAM_AUTHINFO_UNAVAIL)&&cfg.ignore_authinfo_unavail)
-    rc=PAM_IGNORE;
-  else if ((rc==PAM_USER_UNKNOWN)&&cfg.ignore_unknown_user)
-    rc=PAM_IGNORE;
-  if ((rc!=PAM_SUCCESS)&&(rc!=PAM_IGNORE))
-    if (!cfg.no_warn)
-      pam_error(pamh,"LDAP %s session failed",
-                     (action==NSLCD_ACTION_PAM_SESS_O)?"open":"clode");
-  return rc;
+               (action==NSLCD_ACTION_PAM_SESS_O)?"open":"close",ctx->sessid);
+  return PAM_SUCCESS;
 }
 
 /* PAM session open call */
@@ -533,18 +551,36 @@ int pam_sm_chauthtok(pam_handle_t *pamh,int flags,int argc,const char **argv)
   const char *username,*service;
   const char *oldpassword=NULL,*newpassword=NULL;
   struct passwd *pwent;
+  uid_t myuid;
   /* set up configuration */
-  rc=init(pamh,flags,argc,argv,&cfg,&ctx,&username,&service);
+  cfg_init(pamh,flags,argc,argv,&cfg);
+  rc=init(pamh,&cfg,&ctx,&username,&service);
   if (rc!=PAM_SUCCESS)
-    return rc;
+    return remap_pam_rc(rc,&cfg);
+  /* see if we are dealing with an LDAP user first */
+  if (ctx->dn==NULL)
+  {
+    rc=nslcd_request_exists(pamh,ctx,&cfg,username);
+    if (rc!=PAM_SUCCESS)
+      return remap_pam_rc(rc,&cfg);
+  }
   /* prelimenary check, just see if we can connect to the LDAP server
      and authenticate with the current password */
   if (flags&PAM_PRELIM_CHECK)
   {
     /* see if the user is trying to modify another user's password */
     pwent=getpwnam(username);
-    if ((pwent!=NULL)&&(pwent->pw_uid!=getuid()))
+    myuid=getuid();
+    if ((pwent!=NULL)&&(pwent->pw_uid!=myuid)&&(!(flags&PAM_CHANGE_EXPIRED_AUTHTOK)))
     {
+      /* we are root so we can test if nslcd will allow us to change the
+         user's password without the admin password */
+      if (myuid==0)
+      {
+        rc=nslcd_request_authc(pamh,ctx,&cfg,"",service,"");
+        if ((rc==PAM_SUCCESS)&&(ctx->authok==PAM_SUCCESS))
+          return pam_set_item(pamh,PAM_OLDAUTHTOK,"");
+      }
       /* try to  authenticate with the LDAP administrator password by passing
          an empty username to the authc request */
       rc=pam_get_authtok(pamh,PAM_OLDAUTHTOK,&oldpassword,"LDAP administrator password: ");
@@ -562,18 +598,27 @@ int pam_sm_chauthtok(pam_handle_t *pamh,int flags,int argc,const char **argv)
       if (rc!=PAM_SUCCESS)
         return rc;
     }
+    /* check for empty password */
+    if (!cfg.nullok&&((oldpassword==NULL)||(oldpassword[0]=='\0')))
+    {
+      if (cfg.debug)
+        pam_syslog(pamh,LOG_DEBUG,"user has empty password, access denied");
+      return PAM_AUTH_ERR;
+    }
     /* try authenticating */
     rc=nslcd_request_authc(pamh,ctx,&cfg,username,service,oldpassword);
+    if (rc!=PAM_SUCCESS)
+      return remap_pam_rc(rc,&cfg);
+    /* handle authentication result */
+    if (ctx->authok!=PAM_SUCCESS)
+      pam_syslog(pamh,LOG_NOTICE,"%s; user=%s",pam_strerror(pamh,ctx->authok),username);
+    else if (cfg.debug)
+      pam_syslog(pamh,LOG_DEBUG,"authentication succeeded");
+    /* store password (needed if oldpassword was retreived from context) */
     if (rc==PAM_SUCCESS)
-      rc=ctx->authok;
+      return pam_set_item(pamh,PAM_OLDAUTHTOK,oldpassword);
     /* remap error code */
-    if ((rc==PAM_AUTHINFO_UNAVAIL)&&cfg.ignore_authinfo_unavail)
-      rc=PAM_IGNORE;
-    else if ((rc==PAM_USER_UNKNOWN)&&cfg.ignore_unknown_user)
-      rc=PAM_IGNORE;
-    /* TODO: figure out when to return PAM_TRY_AGAIN */
-    /* TODO: if password is incorrect (NSLCD_PAM_AUTH_ERR) log that */
-    return rc;
+    return remap_pam_rc(ctx->authok,&cfg);
   }
   /* get the old password (from the previous call) */
   rc=pam_get_item(pamh,PAM_OLDAUTHTOK,(const void **)&oldpassword);
@@ -590,10 +635,7 @@ int pam_sm_chauthtok(pam_handle_t *pamh,int flags,int argc,const char **argv)
   else
     ctx->authzmsg=(char *)pam_strerror(pamh,rc);
   /* remap error code */
-  if ((rc==PAM_AUTHINFO_UNAVAIL)&&cfg.ignore_authinfo_unavail)
-    rc=PAM_IGNORE;
-  else if ((rc==PAM_USER_UNKNOWN)&&cfg.ignore_unknown_user)
-    rc=PAM_IGNORE;
+  rc=remap_pam_rc(rc,&cfg);
   /* check the returned value */
   if (rc!=PAM_SUCCESS)
   {
