@@ -1,7 +1,7 @@
 
 # alias.py - lookup functions for aliasnet addresses
 #
-# Copyright (C) 2010, 2011 Arthur de Jong
+# Copyright (C) 2010, 2011, 2012 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -18,10 +18,9 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301 USA
 
-import logging
-
-import constants
+import cache
 import common
+import constants
 
 
 attmap = common.Attributes(cn='cn', rfc822MailMember='rfc822MailMember')
@@ -35,17 +34,34 @@ class Search(common.Search):
     required = ('cn', 'rfc822MailMember')
 
 
+class Cache(cache.Cache):
+
+    retrieve_sql = '''
+        SELECT `alias_cache`.`cn` AS `cn`,
+               `alias_1_cache`.`rfc822MailMember` AS `rfc822MailMember`
+        FROM `alias_cache`
+        LEFT JOIN `alias_1_cache`
+          ON `alias_1_cache`.`alias` = `alias_cache`.`cn`
+        '''
+
+    def retrieve(self, parameters):
+        query = cache.Query(self.retrieve_sql, parameters)
+        # return results, returning the members as a list
+        for row in cache.RowGrouper(query.execute(self.con), ('cn', ), ('rfc822MailMember', )):
+            yield row['cn'], row['rfc822MailMember']
+
+
 class AliasRequest(common.Request):
 
-    def write(self, dn, attributes, parameters):
-        # get values
+    def write(self, name, members):
+        self.fp.write_string(name)
+        self.fp.write_stringlist(members)
+
+    def convert(self, dn, attributes, parameters):
         names = attributes['cn']
         members = attributes['rfc822MailMember']
-        # write results
         for name in names:
-            self.fp.write_int32(constants.NSLCD_RESULT_BEGIN)
-            self.fp.write_string(name)
-            self.fp.write_stringlist(members)
+            yield (name, members)
 
 
 class AliasByNameRequest(AliasRequest):
